@@ -1,5 +1,4 @@
-# Copyright (C) 2003, 2004, 2005, 2006, 2008, 2009, 2010 Free Software
-# Foundation, Inc.
+# Copyright (C) 2003, 2004, 2005, 2006  Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +11,9 @@
 # GNU General Public License for more details.
 
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+# 02110-1301, USA.
 
 package Automake::Variable;
 use strict;
@@ -72,7 +73,7 @@ Automake::Variable - support for variable definitions
   # $var->conditions->conds is a list of Automake::Condition.)
   my @conds = $var->conditions->conds
 
-  # Access to the definition in Condition $cond.
+  # Accessing to the definition in Condition $cond.
   # $def is an Automake::VarDef.
   my $def = $var->def ($cond);
   if ($def)
@@ -129,10 +130,7 @@ non-object).
 
 =cut
 
-my $_VARIABLE_CHARACTERS = '[.A-Za-z0-9_@]+';
-my $_VARIABLE_PATTERN = '^' . $_VARIABLE_CHARACTERS . "\$";
-my $_VARIABLE_RECURSIVE_PATTERN =
-    '^([.A-Za-z0-9_@]|\$[({]' . $_VARIABLE_CHARACTERS . '[})]?)+' . "\$";
+my $_VARIABLE_PATTERN = '^[.A-Za-z0-9_@]+' . "\$";
 
 # The order in which variables should be output.  (May contain
 # duplicates -- only the first occurrence matters.)
@@ -162,7 +160,7 @@ my %_am_macro_for_var =
    EMACS => 'AM_PATH_LISPDIR',
    GCJ => 'AM_PROG_GCJ',
    LEX => 'AM_PROG_LEX',
-   LIBTOOL => 'LT_INIT',
+   LIBTOOL => 'AC_PROG_LIBTOOL',
    lispdir => 'AM_PATH_LISPDIR',
    pkgpyexecdir => 'AM_PATH_PYTHON',
    pkgpythondir => 'AM_PATH_PYTHON',
@@ -193,9 +191,9 @@ my %_ac_macro_for_var =
    );
 
 # The name of the configure.ac file.
-my $configure_ac;
+my $configure_ac = find_configure_ac;
 
-# Variables that can be overridden without complaint from -Woverride
+# Variables that can be overriden without complaint from -Woverride
 my %_silent_variable_override =
   (AM_MAKEINFOHTMLFLAGS => 1,
    AR => 1,
@@ -354,7 +352,7 @@ sub reset ()
 =item C<var ($varname)>
 
 Return the C<Automake::Variable> object for the variable
-named C<$varname> if defined.  Return 0 otherwise.
+named C<$varname> if defined.   Return 0 otherwise.
 
 =cut
 
@@ -467,9 +465,8 @@ sub _check_ambiguous_condition ($$$)
   my $var = $self->name;
   my ($message, $ambig_cond) = $self->conditions->ambiguous_p ($var, $cond);
 
-  # We allow silent variables to be overridden silently,
-  # by either silent or non-silent variables.
-  my $def = $self->def ($ambig_cond);
+  # We allow silent variables to be overridden silently.
+  my $def = $self->def ($cond);
   if ($message && !($def && $def->pretty == VAR_SILENT))
     {
       msg 'syntax', $where, "$message ...", partial => 1;
@@ -746,7 +743,7 @@ sub scan_variable_expansions ($)
   while ($text =~ /(?<!\$)\$(?:\{([^\}]*)\}|\(([^\)]*)\))/g)
     {
       my $var = $1 || $2;
-      # The occurrence may look like $(string1[:subst1=[subst2]]) but
+      # The occurence may look like $(string1[:subst1=[subst2]]) but
       # we want only `string1'.
       $var =~ s/:[^:=]*=[^=]*$//;
       push @result, $var;
@@ -776,17 +773,8 @@ sub check_variable_expansions ($$)
 	  # Mention this in the diagnostic.
 	  my $gnuext = "";
 	  $gnuext = "\n(probably a GNU make extension)" if $var =~ / /;
-	  # Accept recursive variable expansions if so desired
-	  # (we hope they are rather portable in practice).
-	  if ($var =~ /$_VARIABLE_RECURSIVE_PATTERN/o)
-	    {
-	      msg ('portability-recursive', $where,
-		   "$var: non-POSIX recursive variable expansion$gnuext");
-	    }
-	  else
-	    {
-	      msg ('portability', $where, "$var: non-POSIX variable name$gnuext");
-	    }
+	  msg ('portability', $where,
+	       "$var: non-POSIX variable name$gnuext");
 	}
     }
 }
@@ -801,7 +789,7 @@ C<$varname>: the name of the variable being defined.
 
 C<$owner>: owner of the variable (one of C<VAR_MAKEFILE>,
 C<VAR_CONFIGURE>, or C<VAR_AUTOMAKE>, defined by L<Automake::VarDef>).
-Variables can be overridden, provided the new owner is not weaker
+Variables can be overriden, provided the new owner is not weaker
 (C<VAR_AUTOMAKE> < C<VAR_CONFIGURE> < C<VAR_MAKEFILE>).
 
 C<$type>: the type of the assignment (C<''> for C<FOO = bar>,
@@ -1132,9 +1120,6 @@ sub require_variables ($$$@)
   my $res = 0;
   $reason .= ' but ' unless $reason eq '';
 
-  $configure_ac = find_configure_ac
-    unless defined $configure_ac;
-
  VARIABLE:
   foreach my $var (@vars)
     {
@@ -1278,13 +1263,13 @@ following arguments:
                    traversing
    $val,        -- the item (i.e., filename) to process
    $cond,       -- the Condition for the $var definition we are
-                   examining (ignoring the recursion context)
+                   examinating (ignoring the recursion context)
    $full_cond)  -- the full Condition, taking into account
                    conditions inherited from parent variables
                    during recursion
 
-If C<inner_expand> is set, variable references occurring in filename
-(as in C<$(BASE).ext>) are expanded before the filename is passed to
+If C<inner_expand> is set, variable references occuring in filename
+(as in C<$(BASE).ext>) are expansed before the filename is passed to
 C<&fun_item>.
 
 If C<skip_ac_subst> is set, Autoconf @substitutions@ will be skipped,
@@ -1504,7 +1489,7 @@ sub _hash_varname ($)
 
 # _hash_values (@VALUES)
 # ----------------------
-# Hash @VALUES for %_gen_varname.  @VALUES should be a list
+# Hash @VALUES for %_gen_varname.  @VALUES shoud be a list
 # of pairs: ([$cond, @values], [$cond, @values], ...).
 # See _gen_varname() below.
 sub _hash_values (@)
@@ -1519,7 +1504,7 @@ sub _hash_values (@)
 }
 # ($VARNAME, $GENERATED)
 # _gen_varname ($BASE, @DEFINITIONS)
-# ----------------------------------
+# ---------------------------------
 # Return a variable name starting with $BASE, that will be
 # used to store definitions @DEFINITIONS.
 # @DEFINITIONS is a list of pair [$COND, @OBJECTS].
